@@ -1,22 +1,25 @@
 /*
  *  C++ Interface to Rserve
- *  Copyright (C) 2004 Simon Urbanek, All rights reserved.
+ *  Copyright (C) 2004-8 Simon Urbanek, All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; version 2.1 of the License
  *
  *  This program is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *  GNU Leser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
+ *  You should have received a copy of the GNU Lesser General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- *  $Id: Rconnection.h 236 2008-08-17 22:01:01Z urbanek $
+ *  Although this code is licensed under LGPL v2.1, we strongly encourage
+ *  everyone modifying this software to contribute back any improvements and
+ *  bugfixes to the project for the benefit all other users. Thank you.
+ *
+ *  $Id: Rconnection.h 265 2009-01-28 20:58:14Z urbanek $
  */
 
 /* external defines:
@@ -113,7 +116,7 @@ public:
 protected:
     // the next two are only cached if requested, no direct access allowed
     int attribs; 
-    char **attrnames;
+    const char **attrnames;
     
     Rexp *master; // if this is set then this Rexp allocated the memory for us, so we are not supposed to free anything; if this is set to "this" then the content is self-allocated, including any data
     int rcount;  // reference count - only for a master - it counts how many children still exist
@@ -132,7 +135,7 @@ public:
     
     virtual void store(char *buf);
     Rexp *attribute(const char *name);
-    char **attributeNames();
+    const char **attributeNames();
     
     virtual Rsize_t length() { return len; }
     
@@ -193,7 +196,7 @@ private:
 
 class Rsymbol : public Rexp {
 protected:
-    char *name;
+    const char *name;
     
 public:
     Rsymbol(Rmessage *msg) : Rexp(msg)
@@ -202,7 +205,7 @@ public:
     Rsymbol(unsigned int *ipos, Rmessage *imsg) : Rexp(ipos, imsg)
     { name=""; fix_content(); }
     
-    char *symbolName() { return name; }
+    const char *symbolName() { return name; }
     
     virtual std::ostream& os_print (std::ostream& os) {
         return os << "Rsymbol[" << symbolName() <<"]";
@@ -226,7 +229,7 @@ public:
     /*Rstring(const char *str) : Rexp(XT_STR, str, strlen(str)+1) {}*/
     
     char **strings() { return cont; }
-    char *stringAt(int i) { return (i<0||i>=nel)?0:cont[i]; }
+    char *stringAt(unsigned int i) { return (i<0||i>=nel)?0:cont[i]; }
     char *string() { return stringAt(0); }
     unsigned int count() { return nel; }
     int indexOfString(const char *str);
@@ -237,7 +240,7 @@ public:
  private:
     void decode() {
       char *c = (char*) data;
-      int i = 0;
+      unsigned int i = 0;
       nel = 0;
       while (i < len) { if (!c[i]) nel++; i++; }
       if (nel) {
@@ -395,6 +398,13 @@ public:
     int writeFile(const char *buf, unsigned int len);
     int closeFile();
     int removeFile(const char *fn);
+	
+#ifdef CMD_ctrl
+    /* server control functions (need Rserve 0.6-0 or higher) */
+    int serverEval(const char *cmd);
+    int serverSource(const char *fn);
+    int serverShutdown();
+#endif
 };
 
 #endif
